@@ -8,35 +8,38 @@ use crate::value::{ApplyMulHandling, BuiltInFunction, Value};
 use std::sync::Arc;
 
 #[derive(Clone, Debug)]
-pub(crate) enum Expr<'a> {
-    Literal(Value<'a>),
-    Ident(Ident<'a>),
-    Parens(Box<Expr<'a>>),
-    UnaryMinus(Box<Expr<'a>>),
-    UnaryPlus(Box<Expr<'a>>),
-    UnaryDiv(Box<Expr<'a>>),
-    Factorial(Box<Expr<'a>>),
-    Add(Box<Expr<'a>>, Box<Expr<'a>>),
-    ImplicitAdd(Box<Expr<'a>>, Box<Expr<'a>>),
-    Sub(Box<Expr<'a>>, Box<Expr<'a>>),
-    Mul(Box<Expr<'a>>, Box<Expr<'a>>),
-    Div(Box<Expr<'a>>, Box<Expr<'a>>),
-    Mod(Box<Expr<'a>>, Box<Expr<'a>>),
-    Pow(Box<Expr<'a>>, Box<Expr<'a>>),
+pub(crate) enum Expr {
+    Literal(Value),
+    Ident(Ident),
+    Parens(Box<Expr>),
+    UnaryMinus(Box<Expr>),
+    UnaryPlus(Box<Expr>),
+    UnaryDiv(Box<Expr>),
+    Factorial(Box<Expr>),
+    Add(Box<Expr>, Box<Expr>),
+    ImplicitAdd(Box<Expr>, Box<Expr>),
+    Sub(Box<Expr>, Box<Expr>),
+    Mul(Box<Expr>, Box<Expr>),
+    Div(Box<Expr>, Box<Expr>),
+    Mod(Box<Expr>, Box<Expr>),
+    Pow(Box<Expr>, Box<Expr>),
     // Call a function or multiply the expressions
-    Apply(Box<Expr<'a>>, Box<Expr<'a>>),
+    Apply(Box<Expr>, Box<Expr>),
     // Call a function, or throw an error if lhs is not a function
-    ApplyFunctionCall(Box<Expr<'a>>, Box<Expr<'a>>),
+    ApplyFunctionCall(Box<Expr>, Box<Expr>),
     // Multiply the expressions
-    ApplyMul(Box<Expr<'a>>, Box<Expr<'a>>),
+    ApplyMul(Box<Expr>, Box<Expr>),
 
-    As(Box<Expr<'a>>, Box<Expr<'a>>),
-    Fn(Ident<'a>, Box<Expr<'a>>),
+    As(Box<Expr>, Box<Expr>),
+    Fn(Ident, Box<Expr>),
 
-    Of(Ident<'a>, Box<Expr<'a>>),
+    Of(Ident, Box<Expr>),
+
+    Assign(Ident, Box<Expr>),
+    Statements(Box<Expr>, Box<Expr>),
 }
 
-impl<'a> Expr<'a> {
+impl<'a> Expr {
     pub(crate) fn format<I: Interrupt>(&self, int: &I) -> Result<String, IntErr<String, I>> {
         Ok(match self {
             Self::Literal(Value::String(s)) => format!(r#""{}""#, s.as_ref()),
@@ -68,12 +71,14 @@ impl<'a> Expr<'a> {
                 }
             }
             Self::Of(a, b) => format!("{} of {}", a, b.format(int)?),
+            Self::Assign(a, b) => format!("{} = {}", a, b.format(int)?),
+            Self::Statements(a, b) => format!("{}; {}", a.format(int)?, b.format(int)?),
         })
     }
 }
 
 /// returns true if rhs is '-1' or '(-1)'
-fn should_compute_inverse(rhs: &Expr<'_>) -> bool {
+fn should_compute_inverse(rhs: &Expr) -> bool {
     if let Expr::UnaryMinus(inner) = &*rhs {
         if let Expr::Literal(Value::Num(n)) = &**inner {
             if n.is_unitless_one() {
@@ -93,12 +98,12 @@ fn should_compute_inverse(rhs: &Expr<'_>) -> bool {
 }
 
 #[allow(clippy::too_many_lines)]
-pub(crate) fn evaluate<'a, I: Interrupt>(
-    expr: Expr<'a>,
-    scope: Option<Arc<Scope<'a>>>,
+pub(crate) fn evaluate<I: Interrupt>(
+    expr: Expr,
+    scope: Option<Arc<Scope>>,
     context: &mut crate::Context,
     int: &I,
-) -> Result<Value<'a>, IntErr<String, I>> {
+) -> Result<Value, IntErr<String, I>> {
     macro_rules! eval {
         ($e:expr) => {
             evaluate($e, scope.clone(), context, int)
@@ -106,28 +111,28 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
     }
     test_int(int)?;
     Ok(match expr {
-        Expr::<'a>::Literal(v) => v,
-        Expr::<'a>::Ident(ident) => resolve_identifier(ident, scope, context, int)?,
-        Expr::<'a>::Parens(x) => eval!(*x)?,
-        Expr::<'a>::UnaryMinus(x) => eval!(*x)?.handle_num(|x| Ok(-x), Expr::UnaryMinus, scope)?,
-        Expr::<'a>::UnaryPlus(x) => eval!(*x)?.handle_num(Ok, Expr::UnaryPlus, scope)?,
-        Expr::<'a>::UnaryDiv(x) => eval!(*x)?.handle_num(
+        Expr::Literal(v) => v,
+        Expr::Ident(ident) => resolve_identifier(&ident, scope, context, int)?,
+        Expr::Parens(x) => eval!(*x)?,
+        Expr::UnaryMinus(x) => eval!(*x)?.handle_num(|x| Ok(-x), Expr::UnaryMinus, scope)?,
+        Expr::UnaryPlus(x) => eval!(*x)?.handle_num(Ok, Expr::UnaryPlus, scope)?,
+        Expr::UnaryDiv(x) => eval!(*x)?.handle_num(
             |x| Number::from(1).div(x, int).map_err(IntErr::into_string),
             Expr::UnaryDiv,
             scope,
         )?,
-        Expr::<'a>::Factorial(x) => {
+        Expr::Factorial(x) => {
             eval!(*x)?.handle_num(|x| x.factorial(int), Expr::Factorial, scope)?
         }
-        Expr::<'a>::Add(a, b) | Expr::<'a>::ImplicitAdd(a, b) => {
+        Expr::Add(a, b) | Expr::ImplicitAdd(a, b) => {
             evaluate_add(eval!(*a)?, eval!(*b)?, scope, int)?
         }
-        Expr::<'a>::Sub(a, b) => {
+        Expr::Sub(a, b) => {
             let a = eval!(*a)?;
             match a {
                 Value::Num(a) => Value::Num(Box::new(a.sub(eval!(*b)?.expect_num()?, int)?)),
-                f @ Value::BuiltInFunction(_) | f @ Value::Fn(_, _, _) => f.apply(
-                    Expr::<'a>::UnaryMinus(b),
+                f @ (Value::BuiltInFunction(_) | Value::Fn(_, _, _)) => f.apply(
+                    Expr::UnaryMinus(b),
                     ApplyMulHandling::OnlyApply,
                     scope,
                     context,
@@ -136,14 +141,14 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
                 _ => return Err("invalid operands for subtraction".to_string().into()),
             }
         }
-        Expr::<'a>::Mul(a, b) => eval!(*a)?.handle_two_nums(
+        Expr::Mul(a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
             |a, b| a.mul(b, int).map_err(IntErr::into_string),
             |a| |f| Expr::Mul(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
             |a| |f| Expr::Mul(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
         )?,
-        Expr::<'a>::Apply(a, b) | Expr::<'a>::ApplyMul(a, b) => {
+        Expr::Apply(a, b) | Expr::ApplyMul(a, b) => {
             if let (Expr::Ident(a), Expr::Ident(b)) = (&*a, &*b) {
                 let ident = format!("{}_{}", a, b);
                 if let Ok(val) = crate::units::query_unit_static(&ident, context, int) {
@@ -152,23 +157,23 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
             }
             eval!(*a)?.apply(*b, ApplyMulHandling::Both, scope, context, int)?
         }
-        Expr::<'a>::Div(a, b) => eval!(*a)?.handle_two_nums(
+        Expr::Div(a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
             |a, b| a.div(b, int).map_err(IntErr::into_string),
             |a| |f| Expr::Div(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
             |a| |f| Expr::Div(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
         )?,
-        Expr::<'a>::Mod(a, b) => eval!(*a)?.handle_two_nums(
+        Expr::Mod(a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
             |a, b| a.modulo(b, int).map_err(IntErr::into_string),
             |a| |f| Expr::Mod(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
             |a| |f| Expr::Mod(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
         )?,
-        Expr::<'a>::Pow(a, b) => {
+        Expr::Pow(a, b) => {
             let lhs = eval!(*a)?;
-            if should_compute_inverse(&*b.clone()) {
+            if should_compute_inverse(&*b) {
                 let result = match &lhs {
                     Value::BuiltInFunction(f) => Some(f.invert()?),
                     Value::Fn(_, _, _) => {
@@ -190,24 +195,33 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
                 scope,
             )?
         }
-        Expr::<'a>::ApplyFunctionCall(a, b) => {
+        Expr::ApplyFunctionCall(a, b) => {
             eval!(*a)?.apply(*b, ApplyMulHandling::OnlyApply, scope, context, int)?
         }
-        Expr::<'a>::As(a, b) => evaluate_as(*a, *b, scope, context, int)?,
-        Expr::<'a>::Fn(a, b) => Value::Fn(a, b, scope),
-        Expr::<'a>::Of(a, b) => match eval!(*b)?.get_object_member(a) {
+        Expr::As(a, b) => evaluate_as(*a, *b, scope, context, int)?,
+        Expr::Fn(a, b) => Value::Fn(a, b, scope),
+        Expr::Of(a, b) => match eval!(*b)?.get_object_member(&a) {
             Ok(value) => value,
             Err(msg) => return Err(msg.into()),
         },
+        Expr::Assign(a, b) => {
+            let rhs = evaluate(*b, scope, context, int)?;
+            context.variables.insert(a.to_string(), rhs.clone());
+            rhs
+        }
+        Expr::Statements(a, b) => {
+            let _lhs = evaluate(*a, scope.clone(), context, int)?;
+            evaluate(*b, scope, context, int)?
+        }
     })
 }
 
-fn evaluate_add<'a, I: Interrupt>(
-    a: Value<'a>,
-    b: Value<'a>,
-    scope: Option<Arc<Scope<'a>>>,
+fn evaluate_add<I: Interrupt>(
+    a: Value,
+    b: Value,
+    scope: Option<Arc<Scope>>,
     int: &I,
-) -> Result<Value<'a>, IntErr<String, I>> {
+) -> Result<Value, IntErr<String, I>> {
     Ok(match (a, b) {
         (Value::Num(a), Value::Num(b)) => Value::Num(Box::new(a.add(*b, int)?)),
         (Value::String(a), Value::String(b)) => {
@@ -235,21 +249,21 @@ fn evaluate_add<'a, I: Interrupt>(
     })
 }
 
-fn evaluate_as<'a, I: Interrupt>(
-    a: Expr<'a>,
-    b: Expr<'a>,
-    scope: Option<Arc<Scope<'a>>>,
+fn evaluate_as<I: Interrupt>(
+    a: Expr,
+    b: Expr,
+    scope: Option<Arc<Scope>>,
     context: &mut crate::Context,
     int: &I,
-) -> Result<Value<'a>, IntErr<String, I>> {
+) -> Result<Value, IntErr<String, I>> {
     if let Expr::Ident(ident) = &b {
         match ident.as_str() {
             "bool" | "boolean" => {
-                let num = evaluate(a, scope.clone(), context, int)?.expect_num()?;
+                let num = evaluate(a, scope, context, int)?.expect_num()?;
                 return Ok((!num.is_zero()).into());
             }
             "date" => {
-                let a = evaluate(a, scope.clone(), context, int)?;
+                let a = evaluate(a, scope, context, int)?;
                 return if let Value::String(s) = a {
                     Ok(crate::date::Date::parse(s.as_ref())
                         .map_err(|e| e.to_string())?
@@ -266,7 +280,7 @@ fn evaluate_as<'a, I: Interrupt>(
                 ));
             }
             "codepoint" => {
-                let a = evaluate(a, scope.clone(), context, int)?;
+                let a = evaluate(a, scope, context, int)?;
                 if let Value::String(s) = a {
                     let ch = s
                         .as_ref()
@@ -333,19 +347,19 @@ fn evaluate_as<'a, I: Interrupt>(
     })
 }
 
-pub(crate) fn resolve_identifier<'a, I: Interrupt>(
-    ident: Ident<'a>,
-    scope: Option<Arc<Scope<'a>>>,
+pub(crate) fn resolve_identifier<I: Interrupt>(
+    ident: &Ident,
+    scope: Option<Arc<Scope>>,
     context: &mut crate::Context,
     int: &I,
-) -> Result<Value<'a>, IntErr<String, I>> {
+) -> Result<Value, IntErr<String, I>> {
     macro_rules! eval_box {
         ($input:expr) => {
             Box::new(evaluate_to_value($input, scope.clone(), context, int)?)
         };
     }
     if let Some(scope) = scope.clone() {
-        match scope.get(ident.as_str(), context, int) {
+        match scope.get(ident, context, int) {
             Ok(val) => return Ok(val),
             Err(IntErr::Interrupt(int)) => return Err(IntErr::Interrupt(int)),
             Err(IntErr::Error(GetIdentError::IdentifierNotFound(_))) => (),
@@ -354,6 +368,9 @@ pub(crate) fn resolve_identifier<'a, I: Interrupt>(
             }
         }
     }
+    if let Some(val) = context.variables.get(ident.as_str()) {
+        return Ok(val.clone());
+    }
     Ok(match ident.as_str() {
         "pi" | "\u{3c0}" => Value::Num(Box::new(Number::pi())),
         "tau" | "\u{3c4}" => Value::Num(Box::new(Number::pi().mul(2.into(), int)?)),
@@ -388,7 +405,7 @@ pub(crate) fn resolve_identifier<'a, I: Interrupt>(
         "auto" => Value::Format(FormattingStyle::Auto),
         "exact" => Value::Format(FormattingStyle::Exact),
         "frac" | "fraction" => Value::Format(FormattingStyle::ImproperFraction),
-        "mixed_fraction" => Value::Format(FormattingStyle::MixedFraction),
+        "mixed_frac" | "mixed_fraction" => Value::Format(FormattingStyle::MixedFraction),
         "float" => Value::Format(FormattingStyle::ExactFloat),
         "dp" => Value::Dp,
         "sf" => Value::Sf,
@@ -401,12 +418,12 @@ pub(crate) fn resolve_identifier<'a, I: Interrupt>(
         "square" => evaluate_to_value("x: x^2", scope, context, int)?,
         "cubic" => evaluate_to_value("x: x^3", scope, context, int)?,
         "earth" => Value::Object(vec![
-            ("axial_tilt", eval_box!("23.4392811 degrees")),
-            ("eccentricity", eval_box!("0.0167086")),
-            ("escape_velocity", eval_box!("11.186 km/s")),
-            ("gravity", eval_box!("9.80665 m/s^2")),
-            ("mass", eval_box!("5.97237e24 kg")),
-            ("volume", eval_box!("1.08321e12 km^3")),
+            ("axial_tilt".into(), eval_box!("23.4392811 degrees")),
+            ("eccentricity".into(), eval_box!("0.0167086")),
+            ("escape_velocity".into(), eval_box!("11.186 km/s")),
+            ("gravity".into(), eval_box!("9.80665 m/s^2")),
+            ("mass".into(), eval_box!("5.97237e24 kg")),
+            ("volume".into(), eval_box!("1.08321e12 km^3")),
         ]),
         "differentiate" => Value::BuiltInFunction(BuiltInFunction::Differentiate),
         "today" => crate::date::Date::today(context)
